@@ -18,12 +18,14 @@ from http import client as http_client
 import inspect
 import io
 import re
+import string
 
 import jsonpatch
 import jsonschema
 from jsonschema import exceptions as json_schema_exc
 import os_traits
 from oslo_config import cfg
+from oslo_policy import policy as oslo_policy
 from oslo_utils import uuidutils
 from pecan import rest
 
@@ -273,6 +275,13 @@ def replace_node_uuid_with_id(to_dict):
         node = objects.Node.get_by_uuid(api.request.context,
                                         to_dict.pop('node_uuid'))
         to_dict['node_id'] = node.id
+        # if they cannot get the node, then this will error
+        # helping guard access to all users of this method as
+        # users which may have rights at a minimum need to be able
+        # to see the node they are trying to do something with.
+        check_owner_policy('node', 'baremetal:node:get', node['owner'],
+                           node['lessee'], conceal_node=node.uuid)
+
     except exception.NodeNotFound as e:
         # Change error code because 404 (NotFound) is inappropriate
         # response for requests acting on non-nodes
@@ -932,6 +941,10 @@ _CONFIG_DRIVE_SCHEMA = {
 }
 
 
+# Include newlines and spaces since they're common in base64 values.
+_B64_ALPHABET = frozenset(string.ascii_letters + string.digits + '+/=\n\r\t ')
+
+
 def check_allow_configdrive(target, configdrive=None):
     if not configdrive:
         return
@@ -969,6 +982,19 @@ def check_allow_configdrive(target, configdrive=None):
                         'opr': versions.MINOR_59_CONFIGDRIVE_VENDOR_DATA}
             raise exception.ClientSideError(
                 msg, status_code=http_client.BAD_REQUEST)
+    else:
+        # : is not a valid base64 symbol, so we can use this simple check
+        if '://' in configdrive:
+            return
+
+        # This is not 100% robust but it does solve the case of invalid
+        # JSON assumed to be a base64 string.
+        letters = set(configdrive)
+        if letters - _B64_ALPHABET:
+            msg = _('Invalid configdrive format: it is neither a JSON, nor '
+                    'a URL, nor a base64 string')
+            raise exception.ClientSideError(
+                msg, status_code=http_client.BAD_REQUEST)
 
 
 def check_allow_filter_by_fault(fault):
@@ -1284,89 +1310,6 @@ def allow_inspect_abort():
     return api.request.version.minor >= versions.MINOR_41_INSPECTION_ABORT
 
 
-def handle_post_port_like_extra_vif(p_dict):
-    """Handle a Post request that sets .extra['vif_port_id'].
-
-    This handles attach of VIFs via specifying the VIF port ID
-    in a port or port group's extra['vif_port_id'] field.
-
-    :param p_dict: a dictionary with field names/values for the port or
-                   port group
-    :return: VIF or None
-    """
-    vif = p_dict.get('extra', {}).get('vif_port_id')
-    if vif:
-        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove
-        #             warning and support for extra[]; else (< 1.28)
-        #             still support it; continue copying to internal_info
-        #             (see bug 1722850). i.e., change the 7 lines of code
-        #             below to something like:
-        #                 if not api_utils.allow_vifs_subcontroller():
-        #                     internal_info = {'tenant_vif_port_id': vif}
-        #                     pg_dict['internal_info'] = internal_info
-        if allow_vifs_subcontroller():
-            utils.warn_about_deprecated_extra_vif_port_id()
-        # NOTE(rloo): this value should really be in .internal_info[..]
-        #             which is what would happen if they had used the
-        #             POST /v1/nodes/<node>/vifs API.
-        internal_info = {'tenant_vif_port_id': vif}
-        p_dict['internal_info'] = internal_info
-    return vif
-
-
-def handle_patch_port_like_extra_vif(rpc_object, internal_info, patch):
-    """Handle a Patch request that modifies .extra['vif_port_id'].
-
-    This handles attach/detach of VIFs via the VIF port ID
-    in a port or port group's extra['vif_port_id'] field.
-
-    :param rpc_object: a Port or Portgroup RPC object
-    :param internal_info: Dict of port or portgroup internal info
-    :param patch: the JSON patch in the API request
-    """
-    vif_list = get_patch_values(patch, '/extra/vif_port_id')
-    vif = None
-    if vif_list:
-        # if specified more than once, use the last value
-        vif = vif_list[-1]
-
-        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove this
-        # warning and don't copy to internal_info; else (<1.28) still
-        # support it; continue copying to internal_info (see bug 1722850).
-        # i.e., change the 8 lines of code below to something like:
-        #   if not allow_vifs_subcontroller():
-        #       int_info = rpc_object.internal_info.get('tenant_vif_port_id')
-        #       if (not int_info or
-        #           int_info == rpc_object.extra.get('vif_port_id')):
-        #           internal_info['tenant_vif_port_id'] = vif
-        if allow_vifs_subcontroller():
-            utils.warn_about_deprecated_extra_vif_port_id()
-        # NOTE(rloo): if the user isn't also using the REST API
-        #             'POST nodes/<node>/vifs', we are safe to copy the
-        #             .extra[] value to the .internal_info location
-        int_info = rpc_object.internal_info.get('tenant_vif_port_id')
-        if (not int_info or int_info == rpc_object.extra.get('vif_port_id')):
-            internal_info['tenant_vif_port_id'] = vif
-
-    elif is_path_removed(patch, '/extra/vif_port_id'):
-        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove this
-        # warning and don't remove from internal_info; else (<1.28) still
-        # support it; remove from internal_info (see bug 1722850).
-        # i.e., change the 8 lines of code below to something like:
-        #   if not allow_vifs_subcontroller():
-        #     int_info = rpc_object.internal_info.get('tenant_vif...')
-        #     if (int_info and int_info==rpc_object.extra.get('vif_port_id')):
-        #         internal_info['tenant_vif_port_id'] = None
-        if allow_vifs_subcontroller():
-            utils.warn_about_deprecated_extra_vif_port_id()
-        # NOTE(rloo): if the user isn't also using the REST API
-        #             'POST nodes/<node>/vifs', we are safe to remove the
-        #             .extra[] value from the .internal_info location
-        int_info = rpc_object.internal_info.get('tenant_vif_port_id')
-        if (int_info and int_info == rpc_object.extra.get('vif_port_id')):
-            internal_info.pop('tenant_vif_port_id')
-
-
 def allow_detail_query():
     """Check if passing a detail=True query string is allowed.
 
@@ -1481,26 +1424,51 @@ def check_policy(policy_name):
     # effectively a noop from an authorization perspective because the values
     # we're comparing are coming from the same place.
     cdict = api.request.context.to_policy_values()
+
     policy.authorize(policy_name, cdict, api.request.context)
 
 
-def check_owner_policy(object_type, policy_name, owner, lessee=None):
+def check_policy_true(policy_name):
+    """Check if the specified policy is authorised for this request.
+
+    :policy_name: Name of the policy to check.
+    :returns: True if policy is matched, otherwise false.
+    """
+    # NOTE(lbragstad): Mapping context attributes into a target dictionary is
+    # effectively a noop from an authorization perspective because the values
+    # we're comparing are coming from the same place.
+    cdict = api.request.context.to_policy_values()
+    return policy.check_policy(policy_name, cdict, api.request.context)
+
+
+def check_owner_policy(object_type, policy_name, owner, lessee=None,
+                       conceal_node=False):
     """Check if the policy authorizes this request on an object.
 
     :param: object_type: type of object being checked
     :param: policy_name: Name of the policy to check.
     :param: owner: the owner
     :param: lessee: the lessee
+    :param: conceal_node: the UUID of the node IF we should
+                          conceal the existence of the node with a
+                          404 Error instead of a 403 Error.
 
     :raises: HTTPForbidden if the policy forbids access.
     """
     cdict = api.request.context.to_policy_values()
-
     target_dict = dict(cdict)
     target_dict[object_type + '.owner'] = owner
     if lessee:
         target_dict[object_type + '.lessee'] = lessee
-    policy.authorize(policy_name, target_dict, api.request.context)
+    try:
+        policy.authorize(policy_name, target_dict, api.request.context)
+    except exception.HTTPForbidden:
+        if conceal_node:
+            # The caller does NOT have access to the node and we've been told
+            # we should return a 404 instead of HTTPForbidden.
+            raise exception.NodeNotFound(node=conceal_node)
+        else:
+            raise
 
 
 def check_node_policy_and_retrieve(policy_name, node_ident,
@@ -1515,20 +1483,30 @@ def check_node_policy_and_retrieve(policy_name, node_ident,
     :raises: NodeNotFound if the node is not found.
     :return: RPC node identified by node_ident
     """
+    conceal_node = False
     try:
         if with_suffix:
             rpc_node = get_rpc_node_with_suffix(node_ident)
         else:
             rpc_node = get_rpc_node(node_ident)
     except exception.NodeNotFound:
-        # don't expose non-existence of node unless requester
-        # has generic access to policy
-        cdict = api.request.context.to_policy_values()
-        policy.authorize(policy_name, cdict, api.request.context)
         raise
-
+    # Project scoped users will get a 404 where as system
+    # scoped should get a 403
+    cdict = api.request.context.to_policy_values()
+    if cdict.get('project_id', False):
+        conceal_node = node_ident
+    try:
+        # Always check the ability to see the node BEFORE anything else.
+        check_owner_policy('node', 'baremetal:node:get', rpc_node['owner'],
+                           rpc_node['lessee'], conceal_node=conceal_node)
+    except exception.NotAuthorized:
+        raise exception.NodeNotFound(node=node_ident)
+    # If we've reached here, we can see the node and we have
+    # access to view it.
     check_owner_policy('node', policy_name,
-                       rpc_node['owner'], rpc_node['lessee'])
+                       rpc_node['owner'], rpc_node['lessee'],
+                       conceal_node=False)
     return rpc_node
 
 
@@ -1545,13 +1523,13 @@ def check_allocation_policy_and_retrieve(policy_name, allocation_ident):
     try:
         rpc_allocation = get_rpc_allocation_with_suffix(
             allocation_ident)
-    except exception.AllocationNotFound:
-        # don't expose non-existence unless requester
-        # has generic access to policy
-        cdict = api.request.context.to_policy_values()
-        policy.authorize(policy_name, cdict, api.request.context)
-        raise
-
+        # If the user is not allowed to view the allocation, then
+        # we need to check that and respond with a 404.
+        check_owner_policy('allocation', 'baremetal:allocation:get',
+                           rpc_allocation['owner'])
+    except exception.NotAuthorized:
+        raise exception.AllocationNotFound(allocation=allocation_ident)
+    # The primary policy check for allocation.
     check_owner_policy('allocation', policy_name, rpc_allocation['owner'])
     return rpc_allocation
 
@@ -1594,7 +1572,9 @@ def check_list_policy(object_type, owner=None):
     try:
         policy.authorize('baremetal:%s:list_all' % object_type,
                          cdict, api.request.context)
-    except exception.HTTPForbidden:
+    except (exception.HTTPForbidden, oslo_policy.InvalidScope):
+        # In the event the scoped policy fails, falling back to the
+        # policy governing a filtered view.
         project_owner = cdict.get('project_id')
         if (not project_owner or (owner and owner != project_owner)):
             raise
@@ -1604,55 +1584,244 @@ def check_list_policy(object_type, owner=None):
     return owner
 
 
-def check_port_policy_and_retrieve(policy_name, port_uuid):
+def check_port_policy_and_retrieve(policy_name, port_ident, portgroup=False):
     """Check if the specified policy authorizes this request on a port.
 
     :param: policy_name: Name of the policy to check.
-    :param: port_uuid: the UUID of a port.
+    :param: port_ident: The name, uuid, or other valid ID value to find
+                        a port or portgroup by.
 
     :raises: HTTPForbidden if the policy forbids access.
     :raises: NodeNotFound if the node is not found.
-    :return: RPC port identified by port_uuid and associated node
+    :return: RPC port identified by port_ident associated node
     """
     context = api.request.context
     cdict = context.to_policy_values()
-
+    owner = None
+    lessee = None
     try:
-        rpc_port = objects.Port.get_by_uuid(context, port_uuid)
-    except exception.PortNotFound:
+        if not portgroup:
+            rpc_port = objects.Port.get(context, port_ident)
+        else:
+            rpc_port = objects.Portgroup.get(context, port_ident)
+    except (exception.PortNotFound, exception.PortgroupNotFound):
         # don't expose non-existence of port unless requester
         # has generic access to policy
-        policy.authorize(policy_name, cdict, context)
         raise
 
-    rpc_node = objects.Node.get_by_id(context, rpc_port.node_id)
     target_dict = dict(cdict)
-    target_dict['node.owner'] = rpc_node['owner']
-    target_dict['node.lessee'] = rpc_node['lessee']
+    try:
+        rpc_node = objects.Node.get_by_id(context, rpc_port.node_id)
+        owner = rpc_node['owner']
+        lessee = rpc_node['lessee']
+    except exception.NodeNotFound:
+        # There is no spoon, err, node.
+        rpc_node = None
+        pass
+    target_dict = dict(cdict)
+    target_dict['node.owner'] = owner
+    target_dict['node.lessee'] = lessee
+    try:
+        policy.authorize('baremetal:node:get', target_dict, context)
+    except exception.NotAuthorized:
+        if not portgroup:
+            raise exception.PortNotFound(port=port_ident)
+        else:
+            raise exception.PortgroupNotFound(portgroup=port_ident)
+
     policy.authorize(policy_name, target_dict, context)
 
     return rpc_port, rpc_node
 
 
-def check_port_list_policy():
+def check_port_list_policy(portgroup=False, parent_node=None,
+                           parent_portgroup=None):
     """Check if the specified policy authorizes this request on a port.
 
+    :param portgroup: Boolean value, default false, indicating if the list
+                      policy check is for a portgroup as the policy names
+                      are different between ports and portgroups.
+    :param parent_node: The UUID of a node, if any, to apply a policy
+                        check to as well before applying other policy
+                        check operations.
+    :param parent_portgroup: The UUID of the parent portgroup if the list
+                             of ports was retrieved via the
+                             /v1/portgroups/<uuid>/ports.
+
     :raises: HTTPForbidden if the policy forbids access.
     :return: owner that should be used for list query, if needed
     """
+
     cdict = api.request.context.to_policy_values()
+
+    # No node is associated with this request, yet.
+    rpc_node = None
+    conceal_linked_node = None
+
+    if parent_portgroup:
+        # lookup the portgroup via the db, and then set parent_node
+        rpc_portgroup = objects.Portgroup.get_by_uuid(api.request.context,
+                                                      parent_portgroup)
+        rpc_node = objects.Node.get_by_id(api.request.context,
+                                          rpc_portgroup.node_id)
+        parent_node = rpc_node.uuid
+
+    if parent_node and not rpc_node:
+        try:
+            rpc_node = objects.Node.get_by_uuid(api.request.context,
+                                                parent_node)
+            conceal_linked_node = rpc_node.uuid
+        except exception.NotFound:
+            # NOTE(TheJulia): This only covers portgroups since
+            # you can't go from ports to other items.
+            raise exception.PortgroupNotFound(portgroup=parent_portgroup)
+
+    if parent_node:
+        try:
+            check_owner_policy(
+                'node', 'baremetal:node:get',
+                rpc_node.owner, rpc_node.lessee,
+                conceal_node=conceal_linked_node)
+        except exception.NotAuthorized:
+            if parent_portgroup:
+                # If this call was invoked with a parent portgroup
+                # then we need to signal the parent portgroup was not
+                # found.
+                raise exception.PortgroupNotFound(
+                    portgroup=parent_portgroup)
+            if parent_node:
+                # This should likely never be hit, because
+                # the existence of a parent node should
+                # trigger the node not found exception to be
+                # explicitly raised.
+                raise exception.NodeNotFound(
+                    node=parent_node)
+            raise
+
     try:
-        policy.authorize('baremetal:port:list_all',
-                         cdict, api.request.context)
+        if not portgroup:
+            policy.authorize('baremetal:port:list_all',
+                             cdict, api.request.context)
+        else:
+            policy.authorize('baremetal:portgroup:list_all',
+                             cdict, api.request.context)
     except exception.HTTPForbidden:
         owner = cdict.get('project_id')
         if not owner:
             raise
-        policy.authorize('baremetal:port:list',
-                         cdict, api.request.context)
+        if not portgroup:
+            policy.authorize('baremetal:port:list',
+                             cdict, api.request.context)
+        else:
+            policy.authorize('baremetal:portgroup:list',
+                             cdict, api.request.context)
         return owner
 
 
+def check_volume_list_policy(parent_node=None):
+    """Check if the specified policy authorizes this request on a volume.
+
+    :param parent_node: The UUID of a node, if any, to apply a policy
+                        check to as well before applying other policy
+                        check operations.
+
+    :raises: HTTPForbidden if the policy forbids access.
+    :return: owner that should be used for list query, if needed
+    """
+
+    cdict = api.request.context.to_policy_values()
+
+    # No node is associated with this request, yet.
+    rpc_node = None
+    conceal_linked_node = None
+
+    if parent_node:
+        try:
+            rpc_node = objects.Node.get_by_uuid(api.request.context,
+                                                parent_node)
+            conceal_linked_node = rpc_node.uuid
+        except exception.NotFound:
+            raise exception.NodeNotFound(node=parent_node)
+    if parent_node:
+        try:
+            check_owner_policy(
+                'node', 'baremetal:node:get',
+                rpc_node.owner, rpc_node.lessee,
+                conceal_node=conceal_linked_node)
+        except exception.NotAuthorized:
+            if parent_node:
+                # This should likely never be hit, because
+                # the existence of a parent node should
+                # trigger the node not found exception to be
+                # explicitly raised.
+                raise exception.NodeNotFound(
+                    node=parent_node)
+            raise
+
+    try:
+        policy.authorize('baremetal:volume:list_all',
+                         cdict, api.request.context)
+    except exception.HTTPForbidden:
+        project_id = cdict.get('project_id')
+        if not project_id:
+            raise
+        policy.authorize('baremetal:volume:list',
+                         cdict, api.request.context)
+        return project_id
+
+
+def check_volume_policy_and_retrieve(policy_name, vol_ident, target=False):
+    """Check if the specified policy authorizes this request on a volume.
+
+    :param: policy_name: Name of the policy to check.
+    :param: vol_ident: The name, uuid, or other valid ID value to find
+                        a volume target or connector by.
+    :param: target: Boolean value to indicate if the check is for a volume
+                    target or connector. Default value is False, implying
+                    connector.
+
+    :raises: HTTPForbidden if the policy forbids access.
+    :raises: VolumeConnectorNotFound if the node is not found.
+    :raises: VolumeTargetNotFound if the node is not found.
+    :return: RPC port identified by port_ident associated node
+    """
+    context = api.request.context
+    cdict = context.to_policy_values()
+    owner = None
+    lessee = None
+    try:
+        if not target:
+            rpc_vol = objects.VolumeConnector.get(context, vol_ident)
+        else:
+            rpc_vol = objects.VolumeTarget.get(context, vol_ident)
+    except (exception.VolumeConnectorNotFound, exception.VolumeTargetNotFound):
+        # don't expose non-existence of volume unless requester
+        # has generic access to policy
+        raise
+
+    target_dict = dict(cdict)
+    try:
+        rpc_node = objects.Node.get_by_id(context, rpc_vol.node_id)
+        owner = rpc_node['owner']
+        lessee = rpc_node['lessee']
+    except exception.NodeNotFound:
+        pass
+    target_dict = dict(cdict)
+    target_dict['node.owner'] = owner
+    target_dict['node.lessee'] = lessee
+    try:
+        policy.authorize('baremetal:node:get', target_dict, context)
+    except exception.NotAuthorized:
+        if not target:
+            raise exception.VolumeConnectorNotFound(connector=vol_ident)
+        else:
+            raise exception.VolumeTargetNotFound(target=vol_ident)
+
+    policy.authorize(policy_name, target_dict, context)
+
+    return rpc_vol, rpc_node
+
+
 def allow_build_configdrive():
     """Check if building configdrive is allowed.
 
@@ -1715,6 +1884,11 @@ def allow_deploy_steps():
     return api.request.version.minor >= versions.MINOR_69_DEPLOY_STEPS
 
 
+def allow_status_in_heartbeat():
+    """Check if heartbeat accepts agent_status and agent_status_message."""
+    return api.request.version.minor >= versions.MINOR_72_HEARTBEAT_STATUS
+
+
 def check_allow_deploy_steps(target, deploy_steps):
     """Check if deploy steps are allowed"""
 
@@ -1734,3 +1908,14 @@ def check_allow_deploy_steps(target, deploy_steps):
                  'provision state to %s or %s') % allowed_states)
         raise exception.ClientSideError(
             msg, status_code=http_client.BAD_REQUEST)
+
+
+def check_allow_clean_disable_ramdisk(target, disable_ramdisk):
+    if disable_ramdisk is None:
+        return
+    elif api.request.version.minor < versions.MINOR_70_CLEAN_DISABLE_RAMDISK:
+        raise exception.NotAcceptable(
+            _("disable_ramdisk is not acceptable in this API version"))
+    elif target != "clean":
+        raise exception.BadRequest(
+            _("disable_ramdisk is supported only with manual cleaning"))
