@@ -88,7 +88,7 @@ _DRAC_BOOT_MODES = ['Bios', 'Uefi']
 _NON_PERSISTENT_BOOT_MODE = 'OneTime'
 
 # Clear job id's constant
-_CLEAR_JOB_IDS = 'JID_CLEARALL_FORCE'
+_CLEAR_JOB_IDS = 'JID_CLEARALL'
 
 # Clean steps constant
 _CLEAR_JOBS_CLEAN_STEPS = ['clear_job_queue', 'known_good_state']
@@ -478,6 +478,17 @@ class DracRedfishManagement(redfish_management.RedfishManagement):
         for (node_uuid, driver, conductor_group,
              driver_internal_info) in node_list:
             try:
+
+                task_monitor_url = driver_internal_info.get(
+                    'import_task_monitor_url')
+                # NOTE(TheJulia): Evaluate if a task montitor URL exists
+                # based upon our inital DB query before pulling a task for
+                # every node in the deployment which reduces the overall
+                # number of DB queries triggering in the background where
+                # no work is required.
+                if not task_monitor_url:
+                    continue
+
                 lock_purpose = 'checking async import configuration task'
                 with task_manager.acquire(context, node_uuid,
                                           purpose=lock_purpose,
@@ -485,10 +496,6 @@ class DracRedfishManagement(redfish_management.RedfishManagement):
                     if not isinstance(task.driver.management,
                                       DracRedfishManagement):
                         continue
-                    task_monitor_url = driver_internal_info.get(
-                        'import_task_monitor_url')
-                    if not task_monitor_url:
-                        continue
                     self._check_import_configuration_task(
                         task, task_monitor_url)
             except exception.NodeNotFound:
@@ -504,7 +511,29 @@ class DracRedfishManagement(redfish_management.RedfishManagement):
         """Checks progress of running import configuration task"""
 
         node = task.node
-        task_monitor = redfish_utils.get_task_monitor(node, task_monitor_url)
+        try:
+            task_monitor = redfish_utils.get_task_monitor(
+                node, task_monitor_url)
+        except exception.RedfishError as e:
+            error_msg = (_("Failed import configuration task: "
+                           "%(task_monitor_url)s. Message: '%(message)s'. "
+                           "Most likely this happened because could not find "
+                           "the task anymore as it got deleted by iDRAC. "
+                           "If not already, upgrade iDRAC firmware to "
+                           "5.00.00.00 or later that preserves tasks for "
+                           "longer or decrease "
+                           "[drac]query_import_config_job_status_interval")
+                         % {'task_monitor_url': task_monitor_url,
+                            'message': e})
+            log_msg = ("Import configuration task failed for node "
+                       "%(node)s. %(error)s" % {'node': task.node.uuid,
+                                                'error': error_msg})
+            info = node.driver_internal_info
+            info.pop('import_task_monitor_url', None)
+            node.driver_internal_info = info
+            node.save()
+            self._set_failed(task, log_msg, error_msg)
+            return
 
         if not task_monitor.is_processing:
             import_task = task_monitor.get_task()
@@ -514,9 +543,21 @@ class DracRedfishManagement(redfish_management.RedfishManagement):
             info.pop('import_task_monitor_url', None)
             node.driver_internal_info = info
 
+            succeeded = False
             if (import_task.task_state == sushy.TASK_STATE_COMPLETED
                 and import_task.task_status in
                     [sushy.HEALTH_OK, sushy.HEALTH_WARNING]):
+
+                # Task could complete with errors (partial success)
+                # iDRAC 5.00.00.00 has stopped reporting Critical messages
+                # so checking also by message_id
+                succeeded = not any(m.message for m in import_task.messages
+                                    if (m.severity
+                                        and m.severity != sushy.SEVERITY_OK)
+                                    or (m.message_id and 'SYS055'
+                                        in m.message_id))
+
+            if succeeded:
                 LOG.info('Configuration import %(task_monitor_url)s '
                          'successful for node %(node)s',
                          {'node': node.uuid,
@@ -546,7 +587,11 @@ class DracRedfishManagement(redfish_management.RedfishManagement):
                 # Select all messages, skipping OEM messages that don't have
                 # `message` field populated.
                 messages = [m.message for m in import_task.messages
-                            if m.message is not None]
+                            if m.message is not None
+                            and ((m.severity
+                                  and m.severity != sushy.SEVERITY_OK)
+                                 or (m.message_id
+                                     and 'SYS055' in m.message_id))]
                 error_msg = (_("Failed import configuration task: "
                                "%(task_monitor_url)s. Message: '%(message)s'.")
                              % {'task_monitor_url': task_monitor_url,
