@@ -21,6 +21,7 @@ from ironic_lib import metrics_utils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import strutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -72,9 +73,22 @@ OPTIONAL_PROPERTIES = {
                         "image containing EFI boot loader. This image will "
                         "be used by ironic when building UEFI-bootable ISO "
                         "out of kernel and ramdisk. Required for UEFI "
-                        "boot from partition images.")
+                        "boot from partition images."),
+    'ilo_add_certificates': _("Boolean value that indicates whether the "
+                              "certificates require to be added to the "
+                              "iLO.")
+}
+KERNEL_PARAM_PROPERTIES = {
+    'ilo_kernel_append_params': _("Additional kernel parameters to pass down "
+                                  "to instance kernel. These parameters can "
+                                  "be consumed by the kernel or by the "
+                                  "applications by reading /proc/cmdline. "
+                                  "Mind severe cmdline size limit. Overrides "
+                                  "[ilo]/kernel_append_params ironic option.")
 }
 COMMON_PROPERTIES = REQUIRED_PROPERTIES
+VMEDIA_OPTIONAL_PROPERTIES = OPTIONAL_PROPERTIES.copy()
+VMEDIA_OPTIONAL_PROPERTIES.update(KERNEL_PARAM_PROPERTIES)
 
 KERNEL_RAMDISK_LABELS = {
     'deploy': REQUIRED_PROPERTIES_UEFI_HTTPS_BOOT,
@@ -123,7 +137,8 @@ def parse_driver_info(node, mode='deploy'):
 
     d_info.update(
         {k: info.get(k, getattr(CONF.conductor, k.replace('ilo_', ''), None))
-         for k in OPTIONAL_PROPERTIES})
+         for k in VMEDIA_OPTIONAL_PROPERTIES})
+    d_info.pop('ilo_add_certificates', None)
 
     return d_info
 
@@ -325,24 +340,6 @@ def prepare_node_for_deploy(task):
             task.node.save()
 
 
-def disable_secure_boot_if_supported(task):
-    """Disables secure boot on node, does not throw if its not supported.
-
-    :param task: a TaskManager instance containing the node to act on.
-    :raises: IloOperationError, if some operation on iLO failed.
-    """
-    try:
-        ilo_common.update_secure_boot_mode(task, False)
-    # We need to handle IloOperationNotSupported exception so that if
-    # the user has incorrectly specified the Node capability
-    # 'secure_boot' to a node that does not have that capability and
-    # attempted deploy. Handling this exception here, will help the
-    # user to tear down such a Node.
-    except exception.IloOperationNotSupported:
-        LOG.warning('Secure boot mode is not supported for node %s',
-                    task.node.uuid)
-
-
 class IloVirtualMediaBoot(base.BootInterface):
 
     capabilities = ['iscsi_volume_boot', 'ramdisk_boot']
@@ -456,6 +453,8 @@ class IloVirtualMediaBoot(base.BootInterface):
             task.node.driver_internal_info['agent_secret_token']
         task.node.save()
 
+        ramdisk_params['boot_method'] = 'vmedia'
+
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
@@ -541,7 +540,7 @@ class IloVirtualMediaBoot(base.BootInterface):
         # Set boot mode
         ilo_common.update_boot_mode(task)
         # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
 
     @METRICS.timer('IloVirtualMediaBoot.clean_up_instance')
     def clean_up_instance(self, task):
@@ -558,7 +557,7 @@ class IloVirtualMediaBoot(base.BootInterface):
         """
         LOG.debug("Cleaning up the instance.")
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
         if (deploy_utils.is_iscsi_boot(task)
             and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
@@ -673,15 +672,14 @@ class IloPXEBoot(pxe.PXEBoot):
         :returns: None
         :raises: IloOperationError, if some operation on iLO failed.
         """
-
         # Set boot mode
         ilo_common.update_boot_mode(task)
-        # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
 
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
         if deploy_utils.is_iscsi_boot(task) and boot_mode == 'uefi':
+            # Need to enable secure boot, if being requested
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             # Need to set 'ilo_uefi_iscsi_boot' param for clean up
             driver_internal_info = task.node.driver_internal_info
             driver_internal_info['ilo_uefi_iscsi_boot'] = True
@@ -711,11 +709,11 @@ class IloPXEBoot(pxe.PXEBoot):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
         driver_internal_info = task.node.driver_internal_info
 
         if (deploy_utils.is_iscsi_boot(task)
-            and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+                and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+            boot_mode_utils.deconfigure_secure_boot_if_needed(task)
             # It will clear iSCSI info from iLO in case of booting from
             # volume in UEFI boot mode
             task.driver.management.clear_iscsi_boot_target(task)
@@ -774,12 +772,11 @@ class IloiPXEBoot(ipxe.iPXEBoot):
 
         # Set boot mode
         ilo_common.update_boot_mode(task)
-        # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
-
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
         if deploy_utils.is_iscsi_boot(task) and boot_mode == 'uefi':
+            # Need to enable secure boot, if being requested
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             # Need to set 'ilo_uefi_iscsi_boot' param for clean up
             driver_internal_info = task.node.driver_internal_info
             driver_internal_info['ilo_uefi_iscsi_boot'] = True
@@ -809,11 +806,11 @@ class IloiPXEBoot(ipxe.iPXEBoot):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
         driver_internal_info = task.node.driver_internal_info
 
         if (deploy_utils.is_iscsi_boot(task)
-            and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+                and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+            boot_mode_utils.deconfigure_secure_boot_if_needed(task)
             # It will clear iSCSI info from iLO in case of booting from
             # volume in UEFI boot mode
             task.driver.management.clear_iscsi_boot_target(task)
@@ -896,6 +893,8 @@ class IloUefiHttpsBoot(base.BootInterface):
         :returns: A dict with the driver_info values.
         :raises: MissingParameterValue, if any of the required parameters are
             missing.
+        :raises: InvalidParameterValue, if any of the required parameters are
+            invalid.
         """
         info = node.driver_info
 
@@ -919,12 +918,30 @@ class IloUefiHttpsBoot(base.BootInterface):
                                     k.replace('ilo_', ''), None))
              for k in OPTIONAL_PROPERTIES})
 
+        should_add_certs = deploy_info.pop('ilo_add_certificates', True)
+
+        if should_add_certs is not None:
+            try:
+                should_add_certs = strutils.bool_from_string(should_add_certs,
+                                                             strict=True)
+            except ValueError:
+                raise exception.InvalidParameterValue(
+                    _('Invalid value type set in driver_info/'
+                      'ilo_add_certificates on node %(node)s. '
+                      'The value should be a Boolean '
+                      ' not "%(value)s"'
+                      ) % {'value': should_add_certs, 'node': node.uuid})
+
         self._validate_hrefs(deploy_info)
 
         error_msg = (_("Error validating %s for iLO UEFI HTTPS boot. Some "
                        "parameters were missing in node's driver_info") % mode)
         deploy_utils.check_for_missing_params(deploy_info, error_msg)
 
+        deploy_info.update(
+            {k: info.get(k, getattr(CONF.ilo, k.replace('ilo_', ''), None))
+             for k in KERNEL_PARAM_PROPERTIES})
+
         deploy_info.update(ilo_common.parse_driver_info(node))
 
         return deploy_info
@@ -1082,6 +1099,9 @@ class IloUefiHttpsBoot(base.BootInterface):
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
 
+        # Signal to IPA that this is a vmedia boot operation.
+        ramdisk_params['boot_method'] = 'vmedia'
+
         mode = 'deploy'
         if node.provision_state == states.RESCUING:
             mode = 'rescue'
@@ -1095,6 +1115,7 @@ class IloUefiHttpsBoot(base.BootInterface):
                   "%(node)s to boot from URL %(iso_ref)s.",
                   {'node': node.uuid, 'iso_ref': iso_ref})
 
+        ilo_common.add_certificates(task)
         ilo_common.setup_uefi_https(task, iso_ref)
 
     @METRICS.timer('IloUefiHttpsBoot.clean_up_ramdisk')
@@ -1144,7 +1165,7 @@ class IloUefiHttpsBoot(base.BootInterface):
                       "%(device)s", {'node': task.node.uuid,
                                      'device': boot_devices.DISK})
             # Need to enable secure boot, if being requested
-            ilo_common.update_secure_boot_mode(task, True)
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             return
 
         params = {}
@@ -1158,7 +1179,7 @@ class IloUefiHttpsBoot(base.BootInterface):
                 manager_utils.node_set_boot_device(task, boot_devices.DISK,
                                                    persistent=True)
                 # Need to enable secure boot, if being requested
-                ilo_common.update_secure_boot_mode(task, True)
+                boot_mode_utils.configure_secure_boot_if_needed(task)
 
                 return
             params.update(root_uuid=root_uuid)
@@ -1173,7 +1194,7 @@ class IloUefiHttpsBoot(base.BootInterface):
             node.save()
 
         # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
         ilo_common.setup_uefi_https(task, iso_ref, persistent=True)
 
         LOG.debug("Node %(node)s is set to boot from UEFIHTTP "
@@ -1193,7 +1214,7 @@ class IloUefiHttpsBoot(base.BootInterface):
                   "%(node)s", {'node': task.node.uuid})
 
         image_utils.cleanup_iso_image(task)
-        disable_secure_boot_if_supported(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
     @METRICS.timer('IloUefiHttpsBoot.validate_rescue')
     def validate_rescue(self, task):
