@@ -891,14 +891,18 @@ class TestNodeIdent(base.TestCase):
         self.assertRaises(exception.NodeNotFound,
                           utils.populate_node_uuid, port, d)
 
+    @mock.patch.object(utils, 'check_owner_policy', autospec=True)
     @mock.patch.object(objects.Node, 'get_by_uuid', autospec=True)
-    def test_replace_node_uuid_with_id(self, mock_gbu, mock_pr):
+    def test_replace_node_uuid_with_id(self, mock_gbu, mock_check, mock_pr):
         node = obj_utils.get_test_node(self.context, id=1)
         mock_gbu.return_value = node
         to_dict = {'node_uuid': self.valid_uuid}
 
         self.assertEqual(node, utils.replace_node_uuid_with_id(to_dict))
         self.assertEqual({'node_id': 1}, to_dict)
+        mock_check.assert_called_once_with('node', 'baremetal:node:get',
+                                           None, None,
+                                           conceal_node=mock.ANY)
 
     @mock.patch.object(objects.Node, 'get_by_uuid', autospec=True)
     def test_replace_node_uuid_with_id_not_found(self, mock_gbu, mock_pr):
@@ -1136,10 +1140,13 @@ class TestCheckNodePolicyAndRetrieve(base.TestCase):
         rpc_node = utils.check_node_policy_and_retrieve(
             'fake_policy', self.valid_node_uuid
         )
+        authorize_calls = [
+            mock.call('baremetal:node:get', expected_target, fake_context),
+            mock.call('fake_policy', expected_target, fake_context)]
+
         mock_grn.assert_called_once_with(self.valid_node_uuid)
         mock_grnws.assert_not_called()
-        mock_authorize.assert_called_once_with(
-            'fake_policy', expected_target, fake_context)
+        mock_authorize.assert_has_calls(authorize_calls)
         self.assertEqual(self.node, rpc_node)
 
     @mock.patch.object(api, 'request', spec_set=["context", "version"])
@@ -1162,14 +1169,16 @@ class TestCheckNodePolicyAndRetrieve(base.TestCase):
         )
         mock_grn.assert_not_called()
         mock_grnws.assert_called_once_with(self.valid_node_uuid)
-        mock_authorize.assert_called_once_with(
-            'fake_policy', expected_target, fake_context)
+        authorize_calls = [
+            mock.call('baremetal:node:get', expected_target, fake_context),
+            mock.call('fake_policy', expected_target, fake_context)]
+        mock_authorize.assert_has_calls(authorize_calls)
         self.assertEqual(self.node, rpc_node)
 
     @mock.patch.object(api, 'request', spec_set=["context"])
     @mock.patch.object(policy, 'authorize', spec=True)
     @mock.patch.object(utils, 'get_rpc_node', autospec=True)
-    def test_check_node_policy_and_retrieve_no_node_policy_forbidden(
+    def test_check_node_policy_and_retrieve_no_node_policy_notfound(
             self, mock_grn, mock_authorize, mock_pr
     ):
         mock_pr.context.to_policy_values.return_value = {}
@@ -1178,7 +1187,7 @@ class TestCheckNodePolicyAndRetrieve(base.TestCase):
             node=self.valid_node_uuid)
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.NodeNotFound,
             utils.check_node_policy_and_retrieve,
             'fake-policy',
             self.valid_node_uuid
@@ -1213,7 +1222,7 @@ class TestCheckNodePolicyAndRetrieve(base.TestCase):
         mock_grn.return_value = self.node
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.NodeNotFound,
             utils.check_node_policy_and_retrieve,
             'fake-policy',
             self.valid_node_uuid
@@ -1244,14 +1253,18 @@ class TestCheckAllocationPolicyAndRetrieve(base.TestCase):
             'fake_policy', self.valid_allocation_uuid
         )
         mock_graws.assert_called_once_with(self.valid_allocation_uuid)
-        mock_authorize.assert_called_once_with(
-            'fake_policy', expected_target, fake_context)
+
+        expected = [mock.call('baremetal:allocation:get',
+                              expected_target, fake_context),
+                    mock.call('fake_policy', expected_target,
+                              fake_context)]
+        mock_authorize.assert_has_calls(expected)
         self.assertEqual(self.allocation, rpc_allocation)
 
     @mock.patch.object(api, 'request', spec_set=["context"])
     @mock.patch.object(policy, 'authorize', spec=True)
     @mock.patch.object(utils, 'get_rpc_allocation_with_suffix', autospec=True)
-    def test_check_alloc_policy_and_retrieve_no_alloc_policy_forbidden(
+    def test_check_alloc_policy_and_retrieve_no_alloc_policy_not_found(
             self, mock_graws, mock_authorize, mock_pr
     ):
         mock_pr.context.to_policy_values.return_value = {}
@@ -1260,7 +1273,7 @@ class TestCheckAllocationPolicyAndRetrieve(base.TestCase):
             allocation=self.valid_allocation_uuid)
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.AllocationNotFound,
             utils.check_allocation_policy_and_retrieve,
             'fake-policy',
             self.valid_allocation_uuid
@@ -1286,7 +1299,7 @@ class TestCheckAllocationPolicyAndRetrieve(base.TestCase):
     @mock.patch.object(api, 'request', spec_set=["context", "version"])
     @mock.patch.object(policy, 'authorize', spec=True)
     @mock.patch.object(utils, 'get_rpc_allocation_with_suffix', autospec=True)
-    def test_check_allocation_policy_and_retrieve_policy_forbidden(
+    def test_check_allocation_policy_and_retrieve_policy_not_found(
             self, mock_graws, mock_authorize, mock_pr
     ):
         mock_pr.version.minor = 50
@@ -1295,7 +1308,7 @@ class TestCheckAllocationPolicyAndRetrieve(base.TestCase):
         mock_graws.return_value = self.allocation
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.AllocationNotFound,
             utils.check_allocation_policy_and_retrieve,
             'fake-policy',
             self.valid_allocation_uuid
@@ -1502,8 +1515,12 @@ class TestCheckPortPolicyAndRetrieve(base.TestCase):
         mock_pgbu.assert_called_once_with(mock_pr.context,
                                           self.valid_port_uuid)
         mock_ngbi.assert_called_once_with(mock_pr.context, 42)
-        mock_authorize.assert_called_once_with(
-            'fake_policy', expected_target, fake_context)
+        expected = [
+            mock.call('baremetal:node:get', expected_target, fake_context),
+            mock.call('fake_policy', expected_target, fake_context)]
+
+        mock_authorize.assert_has_calls(expected)
+
         self.assertEqual(self.port, rpc_port)
         self.assertEqual(self.node, rpc_node)
 
@@ -1519,7 +1536,7 @@ class TestCheckPortPolicyAndRetrieve(base.TestCase):
             port=self.valid_port_uuid)
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.PortNotFound,
             utils.check_port_policy_and_retrieve,
             'fake-policy',
             self.valid_port_uuid
@@ -1546,7 +1563,7 @@ class TestCheckPortPolicyAndRetrieve(base.TestCase):
     @mock.patch.object(policy, 'authorize', spec=True)
     @mock.patch.object(objects.Port, 'get_by_uuid', autospec=True)
     @mock.patch.object(objects.Node, 'get_by_id', autospec=True)
-    def test_check_port_policy_and_retrieve_policy_forbidden(
+    def test_check_port_policy_and_retrieve_policy_notfound(
             self, mock_ngbi, mock_pgbu, mock_authorize, mock_pr
     ):
         mock_pr.version.minor = 50
@@ -1556,7 +1573,7 @@ class TestCheckPortPolicyAndRetrieve(base.TestCase):
         mock_ngbi.return_value = self.node
 
         self.assertRaises(
-            exception.HTTPForbidden,
+            exception.PortNotFound,
             utils.check_port_policy_and_retrieve,
             'fake-policy',
             self.valid_port_uuid
