@@ -31,6 +31,7 @@ from ironic.conductor import base_manager
 from ironic.conductor import manager
 from ironic.conductor import notification_utils
 from ironic.conductor import task_manager
+from ironic.db import api as dbapi
 from ironic.drivers import fake_hardware
 from ironic.drivers import generic
 from ironic.drivers.modules import deploy_utils
@@ -300,6 +301,15 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         mock_zc.close.assert_called_once_with()
         self.assertIsNone(self.service._zeroconf)
 
+    @mock.patch.object(dbapi, 'get_instance', autospec=True)
+    def test_start_dbapi_single_call(self, mock_dbapi):
+        self._start_service()
+        # NOTE(TheJulia): This seems like it should only be 1, but
+        # the hash ring initailization pulls it's own database connection
+        # instance, which is likely a good thing, thus this is 2 instead of
+        # 3 without reuse of the database connection.
+        self.assertEqual(2, mock_dbapi.call_count)
+
 
 class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__check_enabled_interfaces_success(self):
@@ -410,14 +420,37 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         ]
         default_mock.side_effect = ('fake', 'agent', 'fake', 'agent')
         expected_calls = [
-            mock.call(mock.ANY, 'fake-hardware', 'management',
-                      ['fake', 'noop'], 'fake'),
-            mock.call(mock.ANY, 'fake-hardware', 'deploy', ['agent', 'iscsi'],
-                      'agent'),
-            mock.call(mock.ANY, 'manual-management', 'management', ['fake'],
-                      'fake'),
-            mock.call(mock.ANY, 'manual-management', 'deploy',
-                      ['agent', 'fake'], 'agent'),
+            mock.call(
+                mock.ANY,
+                [{'hardware_type': 'fake-hardware',
+                  'interface_type': 'management',
+                  'interface_name': 'fake',
+                  'default': True},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'management',
+                  'interface_name': 'noop',
+                  'default': False},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'deploy',
+                  'interface_name': 'agent',
+                  'default': True},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'deploy',
+                  'interface_name': 'iscsi',
+                  'default': False},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'management',
+                  'interface_name': 'fake',
+                  'default': True},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'deploy',
+                  'interface_name': 'agent',
+                  'default': True},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'deploy',
+                  'interface_name': 'fake',
+                  'default': False}]
+            )
         ]
 
         self.service._register_and_validate_hardware_interfaces(hardware_types)
