@@ -36,10 +36,10 @@ For postgres on Ubuntu this can be done with the following commands:
 
 import collections
 import contextlib
+from unittest import mock
 
 from alembic import script
 import fixtures
-import mock
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import test_fixtures
@@ -65,8 +65,8 @@ MIGRATIONS_TIMEOUT = 300
 
 @contextlib.contextmanager
 def patch_with_engine(engine):
-    with mock.patch.object(enginefacade.writer,
-                           'get_engine') as patch_engine:
+    with mock.patch.object(enginefacade.writer, 'get_engine',
+                           autospec=True) as patch_engine:
         patch_engine.return_value = engine
         yield
 
@@ -150,10 +150,10 @@ class TestWalkVersions(base.TestCase, WalkVersionsMixin):
         self._pre_upgrade_141.assert_called_with(self.engine)
         self._check_141.assert_called_with(self.engine, test_value)
 
-    @mock.patch.object(script, 'ScriptDirectory')
-    @mock.patch.object(WalkVersionsMixin, '_migrate_up')
+    @mock.patch.object(script, 'ScriptDirectory', autospec=True)
+    @mock.patch.object(WalkVersionsMixin, '_migrate_up', autospec=True)
     def test_walk_versions_all_default(self, _migrate_up, script_directory):
-        fc = script_directory.from_config()
+        fc = script_directory.from_config.return_value
         fc.walk_revisions.return_value = self.versions
         self.migration_api.version.return_value = None
 
@@ -161,20 +161,20 @@ class TestWalkVersions(base.TestCase, WalkVersionsMixin):
 
         self.migration_api.version.assert_called_with(self.config)
 
-        upgraded = [mock.call(self.engine, self.config, v.revision,
+        upgraded = [mock.call(self, self.engine, self.config, v.revision,
                     with_data=True) for v in reversed(self.versions)]
         self.assertEqual(self._migrate_up.call_args_list, upgraded)
 
-    @mock.patch.object(script, 'ScriptDirectory')
-    @mock.patch.object(WalkVersionsMixin, '_migrate_up')
+    @mock.patch.object(script, 'ScriptDirectory', autospec=True)
+    @mock.patch.object(WalkVersionsMixin, '_migrate_up', autospec=True)
     def test_walk_versions_all_false(self, _migrate_up, script_directory):
-        fc = script_directory.from_config()
+        fc = script_directory.from_config.return_value
         fc.walk_revisions.return_value = self.versions
         self.migration_api.version.return_value = None
 
         self._walk_versions(self.engine, self.config)
 
-        upgraded = [mock.call(self.engine, self.config, v.revision,
+        upgraded = [mock.call(self, self.engine, self.config, v.revision,
                     with_data=True) for v in reversed(self.versions)]
         self.assertEqual(upgraded, self._migrate_up.call_args_list)
 
@@ -969,6 +969,13 @@ class MigrationCheckersMixin(object):
         col_names = [column.name for column in allocations.c]
         self.assertIn('owner', col_names)
 
+    def _check_cf1a80fdb352(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('network_data', col_names)
+        self.assertIsInstance(
+            nodes.c.network_data.type, sqlalchemy.types.String)
+
     def _pre_upgrade_cd2c80feb331(self, engine):
         data = {
             'node_uuid': uuidutils.generate_uuid(),
@@ -995,6 +1002,11 @@ class MigrationCheckersMixin(object):
         col_names = [column.name for column in nodes.c]
         self.assertIn('lessee', col_names)
 
+    def _check_c0455649680c(self, engine, data):
+        ports = db_utils.get_table(engine, 'ports')
+        col_names = [column.name for column in ports.c]
+        self.assertIn('name', col_names)
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('head')
