@@ -34,6 +34,7 @@ from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.drac import common as drac_common
 from ironic.drivers.modules.drac import job as drac_job
 from ironic.drivers.modules.drac import management as drac_mgmt
+from ironic.drivers.modules.drac import utils as drac_utils
 from ironic.drivers.modules.redfish import utils as redfish_utils
 from ironic.tests.unit.drivers.modules.drac import utils as test_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -813,7 +814,7 @@ class DracManagementTestCase(test_utils.BaseDracTest):
             mock_client.reset_idrac.assert_called_once_with(
                 force=True, wait=True)
             mock_client.delete_jobs.assert_called_once_with(
-                job_ids=['JID_CLEARALL_FORCE'])
+                job_ids=['JID_CLEARALL'])
 
             self.assertIsNone(return_value)
 
@@ -825,7 +826,7 @@ class DracManagementTestCase(test_utils.BaseDracTest):
                                   shared=False) as task:
             return_value = task.driver.management.clear_job_queue(task)
             mock_client.delete_jobs.assert_called_once_with(
-                job_ids=['JID_CLEARALL_FORCE'])
+                job_ids=['JID_CLEARALL'])
 
             self.assertIsNone(return_value)
 
@@ -848,50 +849,6 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.assertRaises(exception.MissingParameterValue,
                           self.management.export_configuration, task, None)
 
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_export_configuration_no_managers(self, mock_get_system):
-        task = mock.Mock(node=self.node, context=self.context)
-        mock_get_system.return_value.managers = []
-
-        self.assertRaises(exception.DracOperationError,
-                          self.management.export_configuration, task, 'edge')
-
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_export_configuration_oem_not_found(self, mock_get_system,
-                                                mock_log):
-        task = mock.Mock(node=self.node, context=self.context)
-        fake_manager1 = mock.Mock()
-        fake_manager1.get_oem_extension.side_effect = (
-            sushy.exceptions.OEMExtensionNotFoundError)
-        mock_get_system.return_value.managers = [fake_manager1]
-
-        self.assertRaises(exception.RedfishError,
-                          self.management.export_configuration, task, 'edge')
-        self.assertEqual(mock_log.error.call_count, 1)
-
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_export_configuration_all_managers_fail(self, mock_get_system,
-                                                    mock_log):
-        task = mock.Mock(node=self.node, context=self.context)
-        fake_manager_oem1 = mock.Mock()
-        fake_manager_oem1.export_system_configuration.side_effect = (
-            sushy.exceptions.SushyError)
-        fake_manager1 = mock.Mock()
-        fake_manager1.get_oem_extension.return_value = fake_manager_oem1
-        fake_manager_oem2 = mock.Mock()
-        fake_manager_oem2.export_system_configuration.side_effect = (
-            sushy.exceptions.SushyError)
-        fake_manager2 = mock.Mock()
-        fake_manager2.get_oem_extension.return_value = fake_manager_oem2
-        mock_get_system.return_value.managers = [fake_manager1, fake_manager2]
-
-        self.assertRaises(exception.DracOperationError,
-                          self.management.export_configuration,
-                          task, 'edge')
-        self.assertEqual(mock_log.debug.call_count, 2)
-
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_export_configuration_export_failed(self, mock_get_system):
         task = mock.Mock(node=self.node, context=self.context)
@@ -905,7 +862,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.assertRaises(exception.DracOperationError,
                           self.management.export_configuration, task, 'edge')
 
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'LOG', autospec=True)
     @mock.patch.object(molds, 'save_configuration', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_export_configuration_success(self, mock_get_system,
@@ -950,66 +907,6 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.assertRaises(exception.DracOperationError,
                           self.management.import_configuration, task, 'edge')
 
-    @mock.patch.object(molds, 'get_configuration', autospec=True)
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_import_configuration_no_managers(self, mock_get_system,
-                                              mock_get_configuration):
-        task = mock.Mock(node=self.node, context=self.context)
-        fake_system = mock.Mock(managers=[])
-        mock_get_configuration.return_value = json.loads(
-            '{"oem": {"interface": "idrac-redfish", '
-            '"data": {"prop1": "value1", "prop2": 2}}}')
-        mock_get_system.return_value = fake_system
-
-        self.assertRaises(exception.DracOperationError,
-                          self.management.import_configuration, task, 'edge')
-
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
-    @mock.patch.object(molds, 'get_configuration', autospec=True)
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_import_configuration_oem_not_found(self, mock_get_system,
-                                                mock_get_configuration,
-                                                mock_log):
-        task = mock.Mock(node=self.node, context=self.context)
-        fake_manager1 = mock.Mock()
-        fake_manager1.get_oem_extension.side_effect = (
-            sushy.exceptions.OEMExtensionNotFoundError)
-        fake_system = mock.Mock(managers=[fake_manager1])
-        mock_get_system.return_value = fake_system
-        mock_get_configuration.return_value = json.loads(
-            '{"oem": {"interface": "idrac-redfish", '
-            '"data": {"prop1": "value1", "prop2": 2}}}')
-
-        self.assertRaises(exception.RedfishError,
-                          self.management.import_configuration, task, 'edge')
-        self.assertEqual(mock_log.error.call_count, 1)
-
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
-    @mock.patch.object(molds, 'get_configuration', autospec=True)
-    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_import_configuration_all_managers_fail(self, mock_get_system,
-                                                    mock_get_configuration,
-                                                    mock_log):
-        task = mock.Mock(node=self.node, context=self.context)
-        fake_manager_oem1 = mock.Mock()
-        fake_manager_oem1.import_system_configuration.side_effect = (
-            sushy.exceptions.SushyError)
-        fake_manager1 = mock.Mock()
-        fake_manager1.get_oem_extension.return_value = fake_manager_oem1
-        fake_manager_oem2 = mock.Mock()
-        fake_manager_oem2.import_system_configuration.side_effect = (
-            sushy.exceptions.SushyError)
-        fake_manager2 = mock.Mock()
-        fake_manager2.get_oem_extension.return_value = fake_manager_oem2
-        mock_get_system.return_value.managers = [fake_manager1, fake_manager2]
-        mock_get_configuration.return_value = json.loads(
-            '{"oem": {"interface": "idrac-redfish", '
-            '"data": {"prop1": "value1", "prop2": 2}}}')
-
-        self.assertRaises(exception.DracOperationError,
-                          self.management.import_configuration, task, 'edge')
-        self.assertEqual(mock_log.debug.call_count, 2)
-
     @mock.patch.object(molds, 'get_configuration', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_import_configuration_incorrect_interface(self, mock_get_system,
@@ -1031,7 +928,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
     @mock.patch.object(deploy_utils, 'set_async_step_flags', autospec=True)
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
-    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'LOG', autospec=True)
     @mock.patch.object(molds, 'get_configuration', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_import_configuration_success(
@@ -1191,6 +1088,25 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             ._check_import_configuration_task
             .assert_called_once_with(task, '/TaskService/123'))
 
+    @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
+    def test__check_import_configuration_task_missing(
+            self, mock_get_task_monitor):
+        mock_get_task_monitor.side_effect = exception.RedfishError(
+            error='Task not found')
+        self.management._set_success = mock.Mock()
+        self.management._set_failed = mock.Mock()
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.management._check_import_configuration_task(
+                task, '/TaskService/123')
+
+            self.management._set_failed.assert_called_once_with(
+                task, mock.ANY, mock.ANY)
+            self.management._set_success.assert_not_called()
+            self.assertIsNone(
+                task.node.driver_internal_info.get('import_task_monitor_url'))
+
     @mock.patch.object(drac_mgmt.LOG, 'debug', autospec=True)
     @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
     def test__check_import_configuration_task_still_processing(
@@ -1252,6 +1168,83 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             self.assertIsNone(
                 task.node.driver_internal_info.get('import_task_monitor_url'))
 
+    @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
+    def test__check_import_configuration_task_partial_failed(
+            self, mock_get_task_monitor):
+        driver_internal_info = {'import_task_monitor_url': '/TaskService/123'}
+        self.node.driver_internal_info = driver_internal_info
+        self.node.save()
+
+        mock_message1 = mock.Mock()
+        mock_message1.message_id = 'SYS413'
+        mock_message1.message = 'The operation successfully completed'
+        mock_message1.severity = sushy.SEVERITY_OK
+        mock_message2 = mock.Mock()
+        mock_message2.message_id = 'SYS055'
+        mock_message2.message = 'Firmware upgrade failed'
+        mock_message2.severity = sushy.SEVERITY_CRITICAL
+        mock_import_task = mock.Mock()
+        mock_import_task.task_state = sushy.TASK_STATE_COMPLETED
+        mock_import_task.task_status = sushy.HEALTH_OK
+        mock_import_task.messages = [mock_message1, mock_message2]
+        mock_task_monitor = mock.Mock()
+        mock_task_monitor.is_processing = False
+        mock_task_monitor.get_task.return_value = mock_import_task
+        mock_get_task_monitor.return_value = mock_task_monitor
+
+        self.management._set_success = mock.Mock()
+        self.management._set_failed = mock.Mock()
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.management._check_import_configuration_task(
+                task, '/TaskService/123')
+
+            self.management._set_failed.assert_called_once_with(
+                task, mock.ANY,
+                "Failed import configuration task: /TaskService/123. Message: "
+                "'Firmware upgrade failed'.")
+            self.management._set_success.assert_not_called()
+            self.assertIsNone(
+                task.node.driver_internal_info.get('import_task_monitor_url'))
+
+    @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
+    def test__check_import_configuration_task_partial_failed_idrac5(
+            self, mock_get_task_monitor):
+        driver_internal_info = {'import_task_monitor_url': '/TaskService/123'}
+        self.node.driver_internal_info = driver_internal_info
+        self.node.save()
+
+        mock_message1 = mock.Mock()
+        mock_message1.message = ('Import of Server Configuration Profile '
+                                 'operation completed with errors')
+        mock_message1.message_id = 'IDRAC.2.4.SYS055'
+        mock_import_task = mock.Mock()
+        mock_import_task.task_state = sushy.TASK_STATE_COMPLETED
+        mock_import_task.task_status = sushy.HEALTH_OK
+        mock_import_task.messages = [mock_message1]
+        mock_task_monitor = mock.Mock()
+        mock_task_monitor.is_processing = False
+        mock_task_monitor.get_task.return_value = mock_import_task
+        mock_get_task_monitor.return_value = mock_task_monitor
+
+        self.management._set_success = mock.Mock()
+        self.management._set_failed = mock.Mock()
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.management._check_import_configuration_task(
+                task, '/TaskService/123')
+
+            self.management._set_failed.assert_called_once_with(
+                task, mock.ANY,
+                "Failed import configuration task: /TaskService/123. Message: "
+                "'Import of Server Configuration Profile "
+                "operation completed with errors'.")
+            self.management._set_success.assert_not_called()
+            self.assertIsNone(
+                task.node.driver_internal_info.get('import_task_monitor_url'))
+
     @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
     def test__check_import_configuration_task(self, mock_get_task_monitor):
         driver_internal_info = {'import_task_monitor_url': '/TaskService/123'}
@@ -1259,7 +1252,9 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.node.save()
 
         mock_message = mock.Mock()
+        mock_message.message_id = 'SYS413'
         mock_message.message = 'Configuration import done'
+        mock_message.severity = sushy.SEVERITY_OK
         mock_import_task = mock.Mock()
         mock_import_task.task_state = sushy.TASK_STATE_COMPLETED
         mock_import_task.task_status = sushy.HEALTH_OK
@@ -1292,7 +1287,9 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.node.save()
 
         mock_message = mock.Mock()
+        mock_message.message_id = 'SYS413'
         mock_message.message = 'Configuration import done'
+        mock_message.severity = sushy.SEVERITY_OK
         mock_import_task = mock.Mock()
         mock_import_task.task_state = sushy.TASK_STATE_COMPLETED
         mock_import_task.task_status = sushy.HEALTH_OK
@@ -1335,7 +1332,9 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.node.save()
 
         mock_message = mock.Mock()
+        mock_message.message_id = 'SYS413'
         mock_message.message = 'Configuration import done'
+        mock_message.severity = sushy.SEVERITY_OK
         mock_import_task = mock.Mock()
         mock_import_task.task_state = sushy.TASK_STATE_COMPLETED
         mock_import_task.task_status = sushy.HEALTH_OK
@@ -1420,7 +1419,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             mock_deploy_handler.assert_called_once_with(
                 task, 'error', 'log message')
 
-    @mock.patch.object(drac_mgmt, 'redfish_utils', autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
     def test_clear_job_queue(self, mock_redfish_utils):
         mock_system = mock_redfish_utils.get_system.return_value
         mock_manager = mock.MagicMock()
@@ -1433,8 +1432,10 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             mock_manager_oem.job_service.delete_jobs.assert_called_once_with(
                 job_ids=['JID_CLEARALL'])
 
-    @mock.patch.object(drac_mgmt, 'redfish_utils', autospec=True)
-    def test_reset_idrac(self, mock_redfish_utils):
+    @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
+                       autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_reset_idrac(self, mock_redfish_utils, mock_wait_system_ready):
         mock_system = mock_redfish_utils.get_system.return_value
         mock_manager = mock.MagicMock()
         mock_system.managers = [mock_manager]
@@ -1445,8 +1446,11 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             task.driver.management.reset_idrac(task)
             mock_manager_oem.reset_idrac.assert_called_once_with()
 
-    @mock.patch.object(drac_mgmt, 'redfish_utils', autospec=True)
-    def test_known_good_state(self, mock_redfish_utils):
+    @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
+                       autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_known_good_state(self, mock_redfish_utils,
+                              mock_wait_system_ready):
         mock_system = mock_redfish_utils.get_system.return_value
         mock_manager = mock.MagicMock()
         mock_system.managers = [mock_manager]
