@@ -16,6 +16,7 @@
 import collections
 import copy
 import os
+import time
 from unittest import mock
 
 from oslo_config import cfg
@@ -202,12 +203,11 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         redfish_utils.get_system(self.node)
         fake_conn.get_system.assert_called_once_with(None)
 
-    @mock.patch('time.sleep', autospec=True)
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     @mock.patch('ironic.drivers.modules.redfish.utils.'
                 'SessionCache._sessions', {})
-    def test_get_system_resource_connection_error_retry(self, mock_sushy,
-                                                        mock_sleep):
+    def test_get_system_resource_connection_error_retry(self, mock_sushy):
         # Redfish specific configurations
         self.config(connection_attempts=3, group='redfish')
 
@@ -223,8 +223,27 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
             mock.call(self.parsed_driver_info['system_id']),
         ]
         fake_conn.get_system.assert_has_calls(expected_get_system_calls)
-        mock_sleep.assert_called_with(
-            redfish_utils.CONF.redfish.connection_retry_interval)
+        self.assertEqual(fake_conn.get_system.call_count,
+                         redfish_utils.CONF.redfish.connection_attempts)
+
+    def test_get_task_monitor(self):
+        redfish_utils._get_connection = mock.Mock()
+        fake_monitor = mock.Mock()
+        redfish_utils._get_connection.return_value = fake_monitor
+        uri = '/redfish/v1/TaskMonitor/FAKEMONITOR'
+
+        response = redfish_utils.get_task_monitor(self.node, uri)
+
+        self.assertEqual(fake_monitor, response)
+
+    def test_get_task_monitor_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        uri = '/redfish/v1/TaskMonitor/FAKEMONITOR'
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.ResourceNotFoundError('GET', uri, mock.Mock())
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_task_monitor, self.node, uri)
 
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     @mock.patch('ironic.drivers.modules.redfish.utils.'
@@ -339,3 +358,20 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
             mock.ANY, verify=mock.ANY,
             auth=mock_basic_auth.return_value
         )
+
+    def test_get_update_service(self):
+        redfish_utils._get_connection = mock.Mock()
+        mock_update_service = mock.Mock()
+        redfish_utils._get_connection.return_value = mock_update_service
+
+        result = redfish_utils.get_update_service(self.node)
+
+        self.assertEqual(mock_update_service, result)
+
+    def test_get_update_service_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.MissingAttributeError
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_update_service, self.node)
