@@ -20,11 +20,14 @@ when needing to mock those third party modules"""
 DRACCLIENT_SPEC = (
     'client',
     'constants',
-    'exceptions'
+    'exceptions',
+    'resources',
+    'utils'
 )
 
 DRACCLIENT_CLIENT_MOD_SPEC = (
     'DRACClient',
+    'WSManClient'
 )
 
 DRACCLIENT_CONSTANTS_MOD_SPEC = (
@@ -46,6 +49,14 @@ DRACCLIENT_CONSTANTS_RAID_STATUS_MOD_SPEC = (
     'raid'
 )
 
+DRACCLIENT_RESOURCES_URIS_MOD_SPEC = (
+    'DCIM_SystemView',
+)
+
+DRACCLIENT_UTILS_MOD_SPEC = (
+    'find_xml',
+)
+
 # proliantutils
 PROLIANTUTILS_SPEC = (
     'exception',
@@ -150,6 +161,9 @@ SUSHY_SPEC = (
     'PROCESSOR_ARCH_ARM',
     'PROCESSOR_ARCH_MIPS',
     'PROCESSOR_ARCH_OEM',
+    'PROTOCOL_TYPE_iSCSI',
+    'PROTOCOL_TYPE_SAS',
+    'PROTOCOL_TYPE_SATA',
     'STATE_ENABLED',
     'STATE_DISABLED',
     'STATE_ABSENT',
@@ -163,7 +177,10 @@ SUSHY_SPEC = (
     'HEALTH_WARNING',
     'SECURE_BOOT_RESET_KEYS_TO_DEFAULT',
     'SECURE_BOOT_RESET_KEYS_DELETE_ALL',
-    'VOLUME_TYPE_RAW_DEVICE'
+    'VOLUME_TYPE_RAW_DEVICE',
+    'SEVERITY_OK',
+    'SEVERITY_WARNING',
+    'SEVERITY_CRITICAL',
 )
 
 SUSHY_AUTH_SPEC = (
