@@ -221,6 +221,9 @@ if not sushy:
         PROCESSOR_ARCH_ARM='ARM',
         PROCESSOR_ARCH_MIPS='MIPS',
         PROCESSOR_ARCH_OEM='OEM-defined',
+        PROTOCOL_TYPE_iSCSI='Internet SCSI',
+        PROTOCOL_TYPE_SAS='Serial Attached SCSI',
+        PROTOCOL_TYPE_SATA='Serial AT Attachment',
         STATE_ENABLED='enabled',
         STATE_DISABLED='disabled',
         STATE_ABSENT='absent',
@@ -234,7 +237,10 @@ if not sushy:
         HEALTH_WARNING='warning',
         SECURE_BOOT_RESET_KEYS_TO_DEFAULT="ResetAllKeysToDefault",
         SECURE_BOOT_RESET_KEYS_DELETE_ALL="DeleteAllKeys",
-        VOLUME_TYPE_RAW_DEVICE='rawdevice'
+        VOLUME_TYPE_RAW_DEVICE='rawdevice',
+        SEVERITY_OK='ok',
+        SEVERITY_WARNING='warning',
+        SEVERITY_CRITICAL='critical'
     )
 
     sys.modules['sushy'] = sushy
